/**
  *  \file planetscentral/fsout.cc
  */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "planetscentral/fsout.h"
#include "server/request.h"

FileServerOutput::FileServerOutput(int fd)
    : fd(fd),
      buf_fill(0)
{ }

void
FileServerOutput::writeSimpleString(string_t str)
{
    post("+", 1);
    post(str.data(), str.size());
    post("\r\n", 2);
}

void
FileServerOutput::writeFullString(string_t str)
{
    char tmp[40];
    sprintf(tmp, "$%ld\r\n", long(str.size()));
    post(tmp, strlen(tmp));
    post(str.data(), str.size());
    post("\r\n", 2);
}

void
FileServerOutput::writeNumber(long number)
{
    char tmp[40];
    sprintf(tmp, ":%ld\r\n", long(number));
    post(tmp, strlen(tmp));
}

void
FileServerOutput::writeArrayStart(long number)
{
    char tmp[40];
    sprintf(tmp, "*%ld\r\n", long(number));
    post(tmp, strlen(tmp));
}

void
FileServerOutput::writeHashStart(long nkeys)
{
    writeArrayStart(2*nkeys);
}

void
FileServerOutput::post(const char* data, size_t size)
{
    while (size > 0) {
        size_t avail = sizeof(buffer) - buf_fill;
        size_t now = size < avail ? size : avail;
        memcpy(&buffer[buf_fill], data, now);
        buf_fill += now;
        size -= now;
        data += now;
        if (buf_fill >= sizeof(buffer)) {
            flush();
        }
    }
}

void
FileServerOutput::flush()
{
    if (buf_fill > 0) {
        write(fd, buffer, buf_fill);
        buf_fill = 0;
    }
}
