/**
  *  \file planetscentral/talk/syntaxnames.cc
  *  \brief Talk: Syntax Highlighter, Name to Syntax
  */

#include "planetscentral/talk/syntaxnames.h"
#include "planetscentral/talk/syntaxc.h"
#include "planetscentral/talk/syntaxini.h"
#include "planetscentral/talk/syntaxdb.h"
#include "planetscentral/talk/syntaxscript.h"

SyntaxDatabase syntax_db;

/** Create a syntax highlighter, given a language/file name hint. */
SyntaxHighlighter*
createHighlighter(string_t name)
{
    // Canonicalize name: lowercase, strip ".frag" used for config fragments
    name = strLCase(name);
    if (name.size() > 5 && name.compare(name.size()-5, 5, ".frag", 5) == 0) {
        name.erase(name.size()-5);
    }

    // Fixed names
    if (name == "pconfig.src" || name == "shiplist.txt") {
        return new SyntaxIni(syntax_db, "phost");
    }
    if (name == "hullfunc.txt") {
        return new SyntaxIni(syntax_db, "hullfunc");
    }
    if (name == "amaster.src") {
        return new SyntaxIni(syntax_db, "amaster");
    }
    if (name == "pmaster.cfg") {
        return new SyntaxIni(syntax_db, "pmaster");
    }
    if (name == "explmap.cfg") {
        return new SyntaxIni(syntax_db, "explmap");
    }
    if (name == "map.ini") {
        return new SyntaxIni(syntax_db, "map");
    }

    // Extensions (and language names)
    string_t::size_type dot = name.rfind('.');
    if (dot != name.npos) {
        name.erase(0, dot+1);
    }
    if (name == "ini" || name == "cfg") {
        return new SyntaxIni(syntax_db, string_t());
    }
    if (name == "q" || name == "ccscript") {
        return new SyntaxScript(syntax_db);
    }
    if (name == "c") {
        return new SyntaxC(SyntaxC::LangC);
    }
    if (name == "c++" || name == "cxx" || name == "cc" || name == "cpp"
        || name == "h++" || name == "hxx" || name == "hh" || name == "hpp"
        || name == "h") {
        return new SyntaxC(SyntaxC::LangCXX);
    }
    if (name == "java") {
        return new SyntaxC(SyntaxC::LangJava);
    }
    if (name == "js" || name == "as" || name == "javascript" || name == "jscript") {
        return new SyntaxC(SyntaxC::LangJavaScript);
    }
    return new NullSyntaxHighlighter();
}
